<?php

namespace Slack\Blockkit\Blocks\Elements;

use Slack\Blockkit\Element;
use Slack\Exceptions\SlackSyntaxException;

final class Confirm extends Element
{
	protected const LIMITS = [
		'title' => 100,
		'text' => 300,
		'confirm' => 30,
		'deny' => 30,
	];

	public function __construct(Text $title,Text $text,Text $confirm,Text $deny)
	{
		parent::__construct();

		if ($title->type != 'plain_text')
			throw new SlackSyntaxException(sprintf('Title must be plain_text not %s',$title->type));

		if ($confirm->type != 'plain_text')
			throw new SlackSyntaxException(sprintf('Title must be plain_text not %s',$confirm->type));

		if ($deny->type != 'plain_text')
			throw new SlackSyntaxException(sprintf('Title must be plain_text not %s',$deny->type));

		$this->title = $this->validate('title',$title->text) ? $title : NULL;

		$this->text = $this->validate('text',$text->text) ? $text : NULL;

		$this->confirm = $this->validate('confirm',$confirm->text) ? $confirm : NULL;

		$this->deny = $this->validate('deny',$deny->text) ? $deny : NULL;
	}

	public static function item(Text $title,Text $text,Text $confirm,Text $deny): self
	{
		return new self($title,$text,$confirm,$deny);
	}

	public function style(string $string): self
	{
		if (! in_array($string,['default','primary','danger']))
			throw new SlackSyntaxException(sprintf('Unknown style %s',$string));

		$this->style = $string;

		return $this;
	}
}