<?php

namespace Slack\Interactive;

use Illuminate\Support\Facades\Log;

use Slack\Blockkit\Modal;

/**
 * Incoming modal view submission event.
 *
 * @package Slack\Interactive
 */
class ViewSubmission extends Base
{
	protected const LOGKEY = 'IVS';

	// View Submissions must respond with via a trigger or inline
	public $respondNow = TRUE;

	public function __get($key)
	{
		switch ($key) {
			case 'blocks':
				return collect(object_get($this->_data,'view.'.$key));

			case 'callback_id':
				return object_get($this->_data,'view.callback_id');

			case 'callback_key':
				return $this->keyitem('id',$this->callback_id);

			case 'callback_value':
				return $this->keyitem('value',$this->callback_id);

			case 'meta':
				return object_get($this->_data,'view.private_metadata');

			case 'state':
				return collect(object_get($this->_data,'view.'.$key.'.values'));

			case 'view_id':
				return object_get($this->_data,'view.id');

			default:
				return parent::__get($key);
		}
	}

	/**
	 * This method should be overridden by a local implementation
	 *
	 * @return Modal
	 */
	public function respond(): Modal
	{
		// Do some magic with event data
		Log::info(sprintf('%s:View Submission for Callback [%s] User [%s] in [%s]',self::LOGKEY,$this->callback_id,$this->user_id,$this->team_id),['m'=>__METHOD__]);
		Log::notice(sprintf('%s:Unhandled action [%s]',self::LOGKEY,$this->callback_key),['m'=>__METHOD__]);

		return new Modal;
	}

	public function value(string $block_id,string $action_id=NULL): ?string
	{
		// If there is no state we need to search out blocks for the block_id
		if (! $this->state->count()) {
			$block = $this->blocks->search(function($item) use ($block_id) { return $item->block_id == $block_id; });
			return $block !== FALSE ? object_get($this->blocks->get($block),$action_id) : NULL;

		} else {
			return object_get($this->state->get($block_id),$action_id.'.value');
		}
	}
}