<?php

namespace Slack\Listeners;

use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Support\Facades\Log;

use Slack\Blockkit\Blocks\Elements\Text;
use Slack\Blockkit\Blocks\{Header,Section};
use Slack\Blockkit\Modal;
use Slack\Interactive\Shortcut;
use Slack\Message;

class ShortcutListener //implements ShouldQueue
{
	protected const LOGKEY = 'LSC';

	public $queue = 'slack';

	/**
	 * Handle the event.
	 *
	 * @param Shortcut $event
	 * @return void
	 * @throws \Exception
	 * @todo To Test
	 */
	public function handle(Shortcut $event): void
	{
		if (! $event->channel() || ! $event->channel()->active) {
			$modal = new Modal(Text::item(config('app.name'),'plain_text'));

			$modal->addBlock(Header::item(Text::item(':robot_face: Bot not in this channel','plain_text')))
				->addBlock(Section::item(Text::item('Please add the BOT to this channel and try this again.')));

			try {
				$event->team()->slackAPI()->viewOpen($event->trigger_id,$modal);

			} catch (\Exception $e) {
				Log::error(sprintf('%s:Got an error posting view to slack: %s',static::LOGKEY,$e->getMessage()),['m'=>__METHOD__]);
			}
		}

		// Do some magic with event data
		Log::info(sprintf('%s:Shortcut [%s] triggered for: [%s]',self::LOGKEY,$event->callback_id,$event->team_id),['m'=>__METHOD__]);
		Log::notice(sprintf('%s:Ignoring Shortcut [%s] in team [%s]',static::LOGKEY,$event->callback_id,$event->team_id),['m'=>__METHOD__]);
	}
}